var Saveable = require('./saveable'),
	_ = require('underscore');
	JsUtils = require('../jsUtils');

module.exports = BaseModel = function (objectExtension) {
	// Do some checks for objectExtension validity
	if (!objectExtension) {
		throw new Error('BaseModel should only be created with an extension object');
	} else if (!objectExtension.collection || typeof objectExtension.collection !== 'string') {
		throw new Error('BaseModel extension should have a collection attribute of a string type');
	}
	
	if (!objectExtension.defaults) {
		objectExtension.defaults = {};
	}
	
	return function (options) {
		// ugly hack to get sub-function callbacks to work
		var that = this;

		this.attributes = {};
		this.toJSON = function () {
			return this.attributes;
		};
		
		this.get = function (index) {
			return this.attributes[index];
		};
		
		this.set = function (index, value) {
			if (index[0] === '_') {
				console.log('WARNING: Setting a model attribute that starts with _. This is potentially VERY dangerous');
			}
			this.attributes[index] = value;
		};
		
		this.setAll = function (keyValues) {
			_.each.call(that, keyValues, function (value, index) {
				that.set(index, value);
			});
		};
		
		_.extend(this.attributes, objectExtension.defaults, options);
		
		// apply normal BaseModel methods and attributes
		Saveable.apply(this, [objectExtension.collection]);
		
		// grab all custom function from the objectExtension and bring it accross
		for (var property in objectExtension) {
			if (typeof objectExtension[property] === 'function') {
				this[property] = objectExtension[property];
			}
		}

		if (this.initialise && typeof this.initialise === 'function') {
			this.initialise(options);
		}
	}
};

