var Api = require('./api');

module.exports = function (options) {
	if (!options.classObject) throw new Error('Cannot create RestApi without specifying a classObject');
	if (!options.classObject.model) throw new Error('Cannot create RestApi without specifying a model');
	if (!options.application) throw new Error('Cannot create RestApi without specifying an application');
	if (!options.path) throw new Error('Cannot create RestApi without specifying a path');

	var app = options.application;
	var classObject = options.classObject;
	var model = options.classObject.model
	var path = options.path + (/\/$/.test(options.path) ? '' : '/');

	// CREATE
	app.post(path, function (req, res) {
		var newObject = new model(req.body);
		newObject.save(function (err, result) {
			if (err) {
				Api.error(res, err);
			} else {
				Api.success(res, result);
			}
		});
	});

	// READ
	app.get(path + ':id', function (req, res) {
		classObject.load(req.params.id, function (err, result) {
			if (err) {
				Api.error(res, err);
			} else {
				Api.success(res, result.toJSON());
			}
		});
	});

	// UPDATE
	app.put(path + ':id', function (req, res) {
		classObject.load(req.params.id, function (err, result) {
			if (err) {
				Api.error(res, err);
			} else {
				result.setAll(req.body);
				result.save(function (err, result) {
					if (err) {
						Api.error(res, err);
					} else {
						Api.success(res, result);
					}
				});
			}
		});
	});

	// DELETE
	app.del(path + ':id', function (req, res) {
		classObject.load(req.params.id, function (err, result) {
			if (err) {
				Api.error(res, err);
			} else {
				result.delete(function (err, result) {
					if (err) {
						Api.error(res, err);
					} else {
						Api.success(res);
					}
				});
			}
		});
	});
};
